import type * as N from "./nodes";
import { Scope } from "./ScopeManager";
export interface IroError {
    message: string;
    location: N.SourceLocation;
    fatal: boolean;
}
export interface TMGrammar {
    name: string;
    scopeName: string;
    firstLineMatch?: string;
    foldingStartMarker?: string;
    foldingStopMarker?: string;
    fileTypes?: string[];
    patterns: TMPattern[];
    repository?: {
        [s: string]: {
            patterns: TMPattern[];
        };
    };
}
export interface TMCaptures {
    [k: string]: {
        name: string;
    };
}
export interface TMPatternSimple {
    name: string;
    match: string;
}
export interface TMPatternCompound {
    match: string;
    captures: TMCaptures;
}
export interface TMPatternComplex {
    begin: string;
    end: string;
    contentName?: string;
    beginCaptures: TMCaptures;
    endCaptures: TMCaptures;
    patterns?: TMPattern[];
}
export interface TMPatternInclude {
    include: string;
}
export type TMPattern = TMPatternSimple | TMPatternCompound | TMPatternComplex | TMPatternInclude;
export interface AceGrammar {
    name: string;
    rules: {
        [s: string]: AcePattern[];
    };
}
export interface AcePatternGeneral {
    token: string | string[];
    regex: string;
    next?: string;
    push?: string;
}
export interface AcePatternDefaultToken {
    defaultToken: string;
}
export interface AcePatternInclude {
    include: string;
}
export type AcePattern = AcePatternGeneral | AcePatternDefaultToken | AcePatternInclude;
export interface Style {
    name: string;
    color?: string;
    textmate: string;
    ace: string;
}
export interface CompileOptions {
    /**Compilation targets */
    targets: ("textmate" | "ace")[];
    /**Emit default token when compiling to ace */
    aceDefaultToken: boolean;
}
export declare function compile(ast: N.Grammar, _options?: Partial<CompileOptions>): {
    errors: IroError[];
    textmate: TMGrammar;
    ace: AceGrammar;
    makeTextmateXML: () => string;
    makeAceHighlighter: (options?: makeAceHighlighterOptions) => string;
    makeCss: () => string;
    styles: Map<string, Style>;
    global: Scope;
    ast: N.Grammar;
};
export interface makeAceHighlighterOptions {
    spacing?: number;
}
export declare function makeAceHighlighter(aceGrammar: AceGrammar, options?: makeAceHighlighterOptions): string;
export declare function makeCss(styles: Map<string, Style>): string;
